﻿
DeclareModule Actions
  
  ; Создает новое действие
  Declare.l CreateAction(callback, shortcut = 0, name$ = "")
  
  ; Изменяет или сбрасывает сочетание клавиш для указанного действия
  Declare.b ChangeActionShortcut(actIndex, newShortcut = 0)
  
  ; Привязывает к заданному пункту меню указанное действие
  Declare.b BindMenuAction(menuItem, actIndex)
  
  ; Инициализирует итерацию по списку действий при помощи функции NextAction()
  Declare.b ExamineActions()
  
  ; Делает текущим очередной элемент списка действий
  Declare.b NextAction()
  
  ; Возвращает индекс текущего элемента списка
  Declare.l ActionIndex()
  
  ; Возвращает для текущего элемента списка имя действия
  Declare.s ActionName()
  
  ; Возвращает для текущего элемента списка символьное отображение горячих клавиш
  Declare.s ActionShortcutString()
  
  ; Загружает сохраненные сочетания клавиш для действий из файла
  Declare LoadActionShortcuts(fileName$ = "")
  
  ; Сохраняет текущие сочетания клавиш для действий в файл
  Declare SaveActionShortcuts(fileName$ = "")
  
EndDeclareModule


Module Actions
  Prototype ActionFunc()
  
  Structure Action
    Call.ActionFunc
    Name.s
    Shortcut.i
    
    IsBound.b
    menuItem.i
  EndStructure
  
  Global NewList lstActions.Action()
  Global NewList lstTmp()
  
  Global shotcutsFileName$ = GetCurrentDirectory() + "shortcuts.dat"
  Global isChanged = #False
  
  ;===================================================================================================
  ; Создает новое действие.
  ;
  ; callback - адрес процедуры обратного вызова
  ; shortcut - сочетание клавиш (#PB_Shortcut_*)
  ; name$    - имя действия для отображения в настройках горячих клавиш
  ;
  ; Возвращает индекс действия в списке, который используется в остальных функциях модуля (параметр actIndex).
  ;===================================================================================================
  Procedure.l CreateAction(callback, shortcut = 0, name$ = "")
    Protected idx
    
    LastElement(lstActions())
    AddElement(lstActions())
    idx = ListIndex(lstActions())
    
    lstActions()\Call = callback
    lstActions()\Name = name$
    lstActions()\IsBound = #False
    
    ChangeActionShortcut(idx, shortcut)    
    
    ProcedureReturn idx
  EndProcedure
  
  ;===================================================================================================
  ; Изменяет или сбрасывает сочетание клавиш для указанного действия.
  ; При при успешном завершении возвращается #True.
  ; Если указанное сочетание клавиш уже используется, то смены сочетания не происходит.
  ;===================================================================================================
  Procedure.b ChangeActionShortcut(actIndex, newShortcut = 0)
    Protected tmp$, pos, oldShortcut
    
    If Not SelectElement(lstActions(), actIndex) Or lstActions()\Shortcut = newShortcut Or Hotkeys::FindHotkey(newShortcut, #True)
      ProcedureReturn #False
    EndIf
      
    oldShortcut = lstActions()\Shortcut
    
    Hotkeys::UpdateHotkeyList(oldShortcut, newShortcut)
      
    lstActions()\Shortcut = newShortcut
    
    isChanged = #True
    
    If lstActions()\IsBound
      tmp$ = GetMenuItemText(Hotkeys::menuID, lstActions()\menuItem)
      If oldShortcut
        ;удаляем отображение предыдущего сочетания клавиш из текста пункта меню
        pos = FindString(tmp$, Chr(9))
        If pos : tmp$ = Left(tmp$, pos - 1) : EndIf
        
        ;отвязываем предыдущее сочетание клавиш от пункта меню
        RemoveKeyboardShortcut(Hotkeys::winID, oldShortcut)
      EndIf
      
      If newShortcut
        ;добавляем отображение нового сочетания клавиш в текст пункта меню
        tmp$ + Chr(9) + Hotkeys::GetShortcutString(lstActions()\Shortcut)
        
        ;привязываем новое сочетание клавиш к пункту меню
        AddKeyboardShortcut(Hotkeys::winID, newShortcut, lstActions()\menuItem)
      EndIf
      
      SetMenuItemText(Hotkeys::menuID, lstActions()\menuItem, tmp$)
    EndIf
    
    ProcedureReturn #True
  EndProcedure
  
  ;===================================================================================================
  ; Привязывает к заданному пункту меню указанное действие.
  ; При удачном завершении возвращается #True.
  ;===================================================================================================
  Procedure.b BindMenuAction(menuItem, actIndex)
    Protected tmp$, pos
    
    If Not Hotkeys::IsInitHotkeys() Or Not SelectElement(lstActions(), actIndex)
      ProcedureReturn #False
    EndIf
    
    lstActions()\IsBound = #True
    lstActions()\menuItem = menuItem
    
    BindMenuEvent(Hotkeys::menuID, menuItem, lstActions()\Call)
    
    If lstActions()\Shortcut
      ;добавляем отображение сочетания клавиш в текст пункта меню
      tmp$ = GetMenuItemText(Hotkeys::menuID, menuItem)
      pos = FindString(tmp$, Chr(9))
      If pos : tmp$ = Left(tmp$, pos - 1) : EndIf
      SetMenuItemText(Hotkeys::menuID, menuItem, tmp$ + Chr(9) + Hotkeys::GetShortcutString(lstActions()\Shortcut))
      
      ;привязываем сочетание клавиш к пункту меню
      AddKeyboardShortcut(Hotkeys::winID, lstActions()\Shortcut, menuItem)
    EndIf
    
    ProcedureReturn #True
  EndProcedure
  
  ;===================================================================================================
  ; Инициализирует итерацию по списку действий при помощи функции NextAction().
  ; Если список действий не пуст, то возвращается #True.
  ;===================================================================================================
  Procedure.b ExamineActions()
    ResetList(lstActions())
    ProcedureReturn Bool(ListSize(lstActions()))
  EndProcedure
  
  ;===================================================================================================
  ; Делает текущим очередной элемент списка действий.
  ; При удачном завершении возвращается #True. Эта функция делает возможным итерацию по списку действий.
  ; Для перемещения указателя в начало списка используйте функцию ExamineActions().
  ;===================================================================================================
  Procedure.b NextAction()
    ProcedureReturn Bool(NextElement(lstActions()))
  EndProcedure
  
  ;===================================================================================================
  ; Возвращает индекс текущего элемента списка.
  ; Если текущий элемент не установлен, то возвращается -1.
  ;===================================================================================================
  Procedure.l ActionIndex()
    ProcedureReturn ListIndex(lstActions())
  EndProcedure
  
  ;===================================================================================================
  ; Возвращает для текущего элемента списка имя действия.
  ; Если текущий элемент не установлен, то возвращается пустая строка.
  ;===================================================================================================
  Procedure.s ActionName()
    If ListIndex(lstActions()) = -1
      ProcedureReturn ""
    Else
      ProcedureReturn lstActions()\Name
    EndIf
  EndProcedure
  
  ;===================================================================================================
  ; Возвращает для текущего элемента списка символьное отображение горячих клавиш.
  ; Если текущий элемент не установлен, то возвращается пустая строка.
  ;===================================================================================================
  Procedure.s ActionShortcutString()
    If ListIndex(lstActions()) = -1
      ProcedureReturn ""
    Else
      ProcedureReturn Hotkeys::GetShortcutString(lstActions()\Shortcut)
    EndIf
  EndProcedure
  
  ;===================================================================================================
  ; Загружает сохраненные сочетания клавиш для действий из файла.
  ; Процедуру имеет смысл вызывать только после создания всех действий.
  ;===================================================================================================
  Procedure LoadActionShortcuts(fileName$ = "")
    If FileSize(fileName$) < 0 
      fileName$ = shotcutsFileName$
    EndIf
    
    If ParseJSON(0, Hotkeys::EncodedFileToJSONString(fileName$)) And JSONType(JSONValue(0)) = #PB_JSON_Array
      ExtractJSONList(JSONValue(0), lstTmp())
      FreeJSON(0)
  
      If ListSize(lstTmp()) = ListSize(lstActions())
        ForEach lstTmp()
          If SelectElement(lstActions(), ListIndex(lstTmp()))
            Hotkeys::UpdateHotkeyList(lstActions()\Shortcut, lstTmp())
            lstActions()\Shortcut = lstTmp()
          EndIf
        Next
      EndIf
      
      ClearList(lstTmp())
      
      isChanged = #False
    EndIf
  EndProcedure
  
  ;===================================================================================================
  ; Сохраняет текущие сочетания клавиш для действий в файл.
  ;===================================================================================================
  Procedure SaveActionShortcuts(fileName$ = "")
    If isChanged
      If fileName$ = ""
        fileName$ = shotcutsFileName$
      EndIf
      
      ForEach lstActions()
        AddElement(lstTmp())
        lstTmp() = lstActions()\Shortcut
      Next
      
      If CreateJSON(0)
        InsertJSONList(JSONValue(0), lstTmp())
        Hotkeys::JSONStringToEncodedFile(fileName$, ComposeJSON(0))
        FreeJSON(0)
      EndIf
      
      ClearList(lstTmp())
      
      isChanged = #False
    EndIf
  EndProcedure
  
EndModule

; IDE Options = PureBasic 5.51 (Windows - x86)
; CursorPosition = 220
; FirstLine = 202
; Folding = ---
; EnableXP