﻿
DeclareModule Options
  ;===================================================================================================
  ; Создает новую настройку соответствующего типа или устанавливает значение существующей настройки.
  ;
  ; key$     - имя настройки
  ; value    - устанавливаемое значение
  ; section$ - имя секции файла настроек, в которой будет размещена данная настройка
  ;===================================================================================================
  Declare SetOptDouble(key$, value.d, section$ = "")
  Declare SetOptQuad(key$, value.q, section$ = "")
  Declare SetOptString(key$, value.s, section$ = "")
  
  ;===================================================================================================
  ; Возвращает текущее значение указанной настройки соответствующего типа.
  ; Настройка должна быть предварительно создана при помощи одной из функций SetOptX.
  ;
  ; key$   - имя настройки
  ;===================================================================================================
  Declare.d GetOptDouble(key$)
  Declare.q GetOptQuad(key$)
  Declare.s GetOptString(key$)
  
  ;===================================================================================================
  ; Считывает значения настроек из указанного файла.
  ;
  ; fileName$     - имя файла, содержащего настройки (если не указано, равно пустой строке или файл не существует, 
  ;                 то будет использован файл "settings.pref" в каталоге приложения)
  ; section$      - имя считываемой секции файла настроек (если не указана или равна пустой строке, 
  ;                 от будут считаны настройки из всех секций файла
  ;
  ; Функция считывает только те настройки, которые были предварительно созданы соответствующей функцией SetOptX.
  ;===================================================================================================
  Declare LoadOptions(fileName$ = "", section$ = "")
  
  ;===================================================================================================
  ; Записывает значения настроек в указанный файл.
  ;
  ; fileName$     - имя файла, содержащего настройки (если не указано, равно пустой строке или файл не существует, 
  ;                 то будет использован файл "settings.pref" в каталоге приложения)
  ; section$      - имя записываемой секции файла настроек (если не указана или равна пустой строке, 
  ;                 от будут записаны настройки всех секций файла
  ;
  ; Функция записывает только те настройки, которые были предварительно созданы соответствующей функцией SetOptX.
  ;===================================================================================================
  Declare SaveOptions(fileName$ = "", section$ = "")
EndDeclareModule


Module Options
  Structure Option
    Section.s
    Type.i
    
    ;соответствующее заданному типу текущее значение
    ValueD.d
    ValueQ.q
    ValueS.s
    
    ;соответствующее заданному типу последнее значение, загруженное из файла
    LoadedD.d
    LoadedQ.q
    LoadedS.s
  EndStructure
  
  Global NewMap mapOptions.Option()
  
  Global settingsFileName$ = GetCurrentDirectory() + "settings.pref"
  
  
  Procedure SetOptDouble(key$, value.d, section$ = "")
    If FindMapElement(mapOptions(), key$) 
      If mapOptions()\Type = #PB_Double
        mapOptions()\ValueD = value
      Else 
        Select mapOptions()\Type
          Case #PB_Quad
            Debug ~"Опция \"" + key$ + ~"\" ожидает значение типа Quad, а не Double"
            mapOptions()\ValueQ = value
          Case #PB_String
            mapOptions()\ValueS = StrD(value)
        EndSelect
      EndIf
    Else
      mapOptions(key$)\Section = section$
      mapOptions(key$)\Type = #PB_Double
      mapOptions(key$)\ValueD = value
      mapOptions(key$)\LoadedD = value
    EndIf
  EndProcedure
  
  
  Procedure SetOptQuad(key$, value.q, section$ = "")
    If FindMapElement(mapOptions(), key$) 
      If mapOptions()\Type = #PB_Quad
        mapOptions()\ValueQ = value
      Else 
        Select mapOptions()\Type
          Case #PB_Double
            Debug ~"Опция \"" + key$ + ~"\" ожидает значение типа Double, а не Quad"
            mapOptions()\ValueD = value
          Case #PB_String
            mapOptions()\ValueS = Str(value)
        EndSelect
      EndIf
    Else
      mapOptions(key$)\Section = section$
      mapOptions(key$)\Type = #PB_Quad
      mapOptions(key$)\ValueQ = value
      mapOptions(key$)\LoadedQ = value
    EndIf
  EndProcedure
  
  
  Procedure SetOptString(key$, value.s, section$ = "")
    If FindMapElement(mapOptions(), key$) 
      If mapOptions()\Type = #PB_String 
        mapOptions()\ValueS = value
      Else 
        Select mapOptions()\Type
          Case #PB_Quad
            Debug ~"Опция \"" + key$ + ~"\" ожидает значение типа Quad, а не String"
            mapOptions()\ValueQ = Val(value)
          Case #PB_Double
            Debug ~"Опция \"" + key$ + ~"\" ожидает значение типа Double, а не String"
            mapOptions()\ValueD = ValD(value)
        EndSelect
      EndIf
    Else
      mapOptions(key$)\Section = section$
      mapOptions(key$)\Type = #PB_String
      mapOptions(key$)\ValueS = value
      mapOptions(key$)\LoadedS = value
    EndIf
  EndProcedure
  
  
  Procedure.d GetOptDouble(key$)
    If FindMapElement(mapOptions(), key$) 
      If mapOptions()\Type = #PB_Double 
        ProcedureReturn mapOptions()\ValueD
      Else 
        Select mapOptions()\Type
          Case #PB_Quad
            Debug ~"Запрашиваемое значение опции \"" + key$ + ~"\" было преобразовано из Quad в Double"
            ProcedureReturn mapOptions()\ValueQ
          Case #PB_String
            Debug ~"Запрашиваемое значение опции \"" + key$ + ~"\" было преобразовано из String в Double"
            ProcedureReturn ValD(mapOptions()\ValueS)
        EndSelect
      EndIf
    Else
      Debug ~"Опция \"" + key$ + ~"\" не найдена"
      ProcedureReturn 0
    EndIf
  EndProcedure
  
  
  Procedure.q GetOptQuad(key$)
    If FindMapElement(mapOptions(), key$) 
      If mapOptions()\Type = #PB_Quad
        ProcedureReturn mapOptions()\ValueQ
      Else 
        Select mapOptions()\Type
          Case #PB_Double
            Debug ~"Запрашиваемое значение опции \"" + key$ + ~"\" было преобразовано из Double в Quad"
            ProcedureReturn mapOptions()\ValueD
          Case #PB_String
            Debug ~"Запрашиваемое значение опции \"" + key$ + ~"\" было преобразовано из String в Quad"
            ProcedureReturn Val(mapOptions()\ValueS)
        EndSelect
      EndIf
    Else
      Debug ~"Опция \"" + key$ + ~"\" не найдена"
      ProcedureReturn 0
    EndIf
  EndProcedure
  
  
  Procedure.s GetOptString(key$)
    If FindMapElement(mapOptions(), key$) 
      If mapOptions()\Type = #PB_String
        ProcedureReturn mapOptions()\ValueS
      Else 
        Select mapOptions()\Type
          Case #PB_Quad
            ProcedureReturn Str(mapOptions()\ValueQ)
          Case #PB_Double
            ProcedureReturn StrD(mapOptions()\ValueD)
        EndSelect
      EndIf
    Else
      Debug ~"Опция \"" + key$ + ~"\" не найдена"
      ProcedureReturn ""
    EndIf
  EndProcedure
  
  
  Procedure LoadOptions(fileName$ = "", section$ = "")
    If fileName$ = "" Or FileSize(fileName$) < 0 
      fileName$ = settingsFileName$
    EndIf
    
    If OpenPreferences(fileName$, #PB_Preference_NoSpace | #PB_Preference_GroupSeparator)
      
      ForEach mapOptions()
        If section$ <> "" And mapOptions()\Section <> section$ : Continue : EndIf
        
        PreferenceGroup(mapOptions()\Section)
        Select mapOptions()\Type
          Case #PB_Double
            mapOptions()\ValueD = ReadPreferenceDouble(MapKey(mapOptions()), mapOptions()\LoadedD)
            mapOptions()\LoadedD = mapOptions()\ValueD
          Case #PB_Quad
            mapOptions()\ValueQ = ReadPreferenceQuad(MapKey(mapOptions()), mapOptions()\LoadedQ)
            mapOptions()\LoadedQ = mapOptions()\ValueQ
          Case #PB_String
            mapOptions()\ValueS = ReadPreferenceString(MapKey(mapOptions()), mapOptions()\LoadedS)
            mapOptions()\LoadedS = mapOptions()\ValueS
        EndSelect
      Next
      
      ClosePreferences()
    EndIf
  EndProcedure
  
  
  Procedure SaveOptions(fileName$ = "", section$ = "")
    If fileName$ = "" Or FileSize(fileName$) < 0 
      fileName$ = settingsFileName$
    EndIf
    
    OpenPreferences(fileName$, #PB_Preference_NoSpace | #PB_Preference_GroupSeparator)
    
    ForEach mapOptions()
      If section$ <> "" And mapOptions()\Section <> section$ : Continue : EndIf
      
      PreferenceGroup(mapOptions()\Section)
      Select mapOptions()\Type
        Case #PB_Double
          If mapOptions()\ValueD <> ReadPreferenceDouble(MapKey(mapOptions()), mapOptions()\LoadedD)
            WritePreferenceDouble(MapKey(mapOptions()), mapOptions()\ValueD)
          EndIf
        Case #PB_Quad
          If mapOptions()\ValueQ <> ReadPreferenceQuad(MapKey(mapOptions()), mapOptions()\LoadedQ)
            WritePreferenceQuad(MapKey(mapOptions()), mapOptions()\ValueQ)
          EndIf
        Case #PB_String
          If mapOptions()\ValueS <> ReadPreferenceString(MapKey(mapOptions()), mapOptions()\LoadedS)
            WritePreferenceString(MapKey(mapOptions()), mapOptions()\ValueS)
          EndIf
      EndSelect
    Next
    
    ClosePreferences()
  EndProcedure
EndModule


; IDE Options = PureBasic 5.51 (Windows - x86)
; CursorPosition = 179
; FirstLine = 143
; Folding = --
; EnableXP