﻿
DeclareModule Plugins
  ; Загружает все доступные плагины
  Declare.i LoadPlugins(pluginsDir$)
  
  ; Создает в текущем меню пункты для вызова функций загруженных плагинов
  Declare AddPluginsToMenu(mainMenuID)
  
  ; Выгружает все загруженные плагины
  Declare CloseAllPlugins()
  
  ; Проверяет, не выбран ли один из пунктов меню плагинов
  Declare.b IsPluginMenuEvent(eventMenu)
  
  ; Вызывает функцию плагина, соответствующую выбранному пункту меню
  Declare CallPluginFunction(eventMenu)
  
  ; Заполняет указанный ListIconGadget информацией о доступных плагинах
  Declare FillListPlugin(listIconID)  
EndDeclareModule


Module Plugins
  ;Стартовое значение для идентификаторов, используемых в системе плагинов.
  ;Оно должно быть больше любых других идентификаторов пунктов меню, гаджетов и т.п.
  #PluginID_First = 10000
  #PluginID_Last = 19999
  
  ;Шаг для идентификаторов плагинов.
  ;Он должен быть больше максимально возможного количества функций, экспортируемых плагином.
  #PluginID_Step = 100
  
  ;прототип функций плагинов
  Prototype.l PluginFunc(Param1.l, Param2.l = 0, Param3.l = 0, Param4.l = 0)
  
  Structure Function
    Name.s
    MenuItemName.s
    Call.PluginFunc
  EndStructure
  
  Structure Plugin
    FullPath.s
    Name.s
    Version.s
    Description.s
    Map Functions.Function()
  EndStructure
  
  Global NewMap mapPlugins.Plugin()
  Global mainMenu
  
  ; Выгружает плагин с указанным идентификатором. Из карты удаляется информация о выгруженном плагине. 
  ; При этом пункты меню для вызова функций плагина не удаляются, но становятся недоступными.
  Declare ClosePlugin(PluginID.i)
  
  XIncludeFile "shared.pbi"
  
  ;===================================================================================================
  ; Загружает все доступные плагины.
  ; При этом соответствующая карта заполняется информацией о каждом корректно загруженном плагине.
  ;
  ; pluginsDir$  - путь к каталогу, содержащему плагины
  ;
  ; В случае успешного завершения возвращается количество загруженных плагинов, в противном случае 0.
  ;===================================================================================================
  Procedure.i LoadPlugins(pluginsDir$)
    Protected funcName$
    Protected plugID = #PluginID_First
    Protected newMenuID = #PluginID_First + 1
    
    ClearMap(mapPlugins())
    If ExamineDirectory(0, pluginsDir$, "*.dll")
      While NextDirectoryEntry(0)
        If DirectoryEntryType(0) = #PB_DirectoryEntry_File
          If OpenLibrary(plugID, pluginsDir$ + DirectoryEntryName(0))
            If ParseJSON(0, PeekS(CallFunction(plugID, "GetPluginInfo")))
              ExtractJSONStructure(JSONValue(0), @p.PluginInfo, PluginInfo)
              
              AddMapElement(mapPlugins(), Str(plugID))
              mapPlugins()\FullPath = pluginsDir$ + DirectoryEntryName(0)
              mapPlugins()\Name = p\Name
              mapPlugins()\Version = p\Version
              mapPlugins()\Description = p\Description
              
              If ExamineLibraryFunctions(plugID)
                While NextLibraryFunction()
                  funcName$ = LibraryFunctionName()
                  Select LCase(funcName$)
                    Case "getplugininfo"
                      ;эту функцию игнорируем, она уже была вызвана
                      
                    Case "plugininit"
                      CallFunction(plugID, funcName$)
                      
                    Case "pluginfinal"
                      ;эту функцию игнорируем, она будет вызвана автоматически перед выгрузкой плагина
                      
                    Default
                      AddMapElement(mapPlugins()\Functions(), Str(newMenuID))
                      mapPlugins()\Functions()\Name = LibraryFunctionName()
                      mapPlugins()\Functions()\Call = LibraryFunctionAddress()
                      If FindMapElement(p\Functions(), LibraryFunctionName())
                        mapPlugins()\Functions()\MenuItemName = p\Functions()
                      EndIf
                      newMenuID + 1
                  EndSelect
                Wend
              EndIf
              plugID + #PluginID_Step
              newMenuID = plugID + 1
            EndIf
          EndIf
        EndIf
      Wend
      
      ProcedureReturn MapSize(mapPlugins())
    EndIf
    
    ProcedureReturn 0
  EndProcedure
  
  ;===================================================================================================
  ; Создает в текущем меню пункты для вызова функций загруженных плагинов.
  ;===================================================================================================
  Procedure AddPluginsToMenu(mainMenuID)
    If IsMenu(mainMenuID)
      mainMenu = mainMenuID
      
      If MapSize(mapPlugins())
        ForEach mapPlugins()
          ForEach mapPlugins()\Functions()
            MenuItem(Val(MapKey(mapPlugins()\Functions())), mapPlugins()\Functions()\MenuItemName)
          Next
        Next
      EndIf
    EndIf
  EndProcedure
  
  ;===================================================================================================
  ; Выгружает все загруженные плагины.
  ; Карта с информацией о загруженных плагинах очищается. При этом пункты меню для вызова функций плагинов
  ; не удаляются, но становятся недоступными.
  ;===================================================================================================
  Procedure CloseAllPlugins()
    ForEach mapPlugins()
      ClosePlugin(Val(MapKey(mapPlugins())))
    Next
  EndProcedure
  
  ;===================================================================================================
  ; Проверяет, не выбран ли один из пунктов меню плагинов.
  ;===================================================================================================
  Procedure.b IsPluginMenuEvent(eventMenu)
    ProcedureReturn Bool(eventMenu >= #PluginID_First And eventMenu <= #PluginID_Last)
  EndProcedure
  
  ;===================================================================================================
  ; Вызывает функцию плагина, соответствующую выбранному пункту меню.
  ;===================================================================================================
  Procedure CallPluginFunction(eventMenu)
    ForEach mapPlugins()
      ForEach mapPlugins()\Functions()
        If eventMenu = Val(MapKey(mapPlugins()\Functions()))
          mapPlugins()\Functions()\Call(@"test string", @"yes")
        EndIf
      Next
    Next
  EndProcedure
  
  ;===================================================================================================
  ; Заполняет указанный ListIconGadget информацией о доступных плагинах.
  ; Гаджет должен иметь 3 колонки, в которые записывается имя плагина, его версия и краткое описание.
  ;===================================================================================================
  Procedure FillListPlugin(listIconID)
    If IsGadget(listIconID) And GadgetType(listIconID) = #PB_GadgetType_ListIcon
      ForEach mapPlugins()
        AddGadgetItem(listIconID, -1, mapPlugins()\Name + Chr(10) + mapPlugins()\Version + Chr(10) + mapPlugins()\Description)
      Next
    EndIf
  EndProcedure
  
  ;===================================================================================================  
  ; Выгружает плагин с указанным идентификатором. Из карты удаляется информация о выгруженном плагине. 
  ; При этом пункты меню для вызова функций плагина не удаляются, но становятся недоступными.
  ;===================================================================================================
  Procedure ClosePlugin(PluginID.i)
    Protected key$ = Str(PluginID)
    
    If FindMapElement(mapPlugins(), key$)
      ForEach mapPlugins()\Functions()
        DeleteMenu_(MenuID(mainMenu), Val(MapKey(mapPlugins()\Functions())), #MF_BYCOMMAND)
      Next
    EndIf
    CallFunction(PluginID, "PluginFinal")
    CloseLibrary(PluginID)
    DeleteMapElement(mapPlugins(), key$)
  EndProcedure
EndModule
; IDE Options = PureBasic 5.51 (Windows - x86)
; Folding = --
; EnableXP